@@ -46,36 +46,36 @@
 #include <mach.h>
 #include <sys/ioctl.h>
 
-#define DEFAULT_MOUSE_DEV	"/dev/mouse"
+#define DEFAULT_MOUSE_DEV       "/dev/mouse"
 
-typedef unsigned short kev_type;		/* kd event type */
+typedef unsigned short kev_type;                /* kd event type */
 typedef unsigned char Scancode;
 
-struct mouse_motion {		
-    short mm_deltaX;		/* units? */
+struct mouse_motion {
+    short mm_deltaX;            /* units? */
     short mm_deltaY;
 };
 
 typedef struct {
-    kev_type type;			/* see below */
-    struct timeval time;		/* timestamp */
-    union {				/* value associated with event */
-	boolean_t up;		/* MOUSE_LEFT .. MOUSE_RIGHT */
-	Scancode sc;		/* KEYBD_EVENT */
-	struct mouse_motion mmotion;	/* MOUSE_MOTION */
+    kev_type type;                      /* see below */
+    struct timeval time;                /* timestamp */
+    union {                             /* value associated with event */
+        boolean_t up;           /* MOUSE_LEFT .. MOUSE_RIGHT */
+        Scancode sc;            /* KEYBD_EVENT */
+        struct mouse_motion mmotion;    /* MOUSE_MOTION */
     } value;
 } kd_event;
 
-/* 
+/*
  * kd_event ID's.
  */
-#define MOUSE_LEFT	1		/* mouse left button up/down */
-#define MOUSE_MIDDLE	2
-#define MOUSE_RIGHT	3
-#define MOUSE_MOTION	4		/* mouse motion */
-#define KEYBD_EVENT	5		/* key up/down */
+#define MOUSE_LEFT      1               /* mouse left button up/down */
+#define MOUSE_MIDDLE    2
+#define MOUSE_RIGHT     3
+#define MOUSE_MOTION    4               /* mouse motion */
+#define KEYBD_EVENT     5               /* key up/down */
 
-#define NUMEVENTS	64
+#define NUMEVENTS       64
 
 /*
  * OsMouseReadInput --
@@ -87,7 +87,7 @@ OsMouseReadInput(InputInfoPtr pInfo)
     MouseDevPtr pMse;
     static kd_event eventList[NUMEVENTS];
     static int remainder = 0;
-    int n, c; 
+    int n, c;
     kd_event *event = eventList;
     unsigned char *pBuf;
 
@@ -97,36 +97,36 @@ OsMouseReadInput(InputInfoPtr pInfo)
     pBuf = (unsigned char *)eventList;
     n = remainder;
     while (n < sizeof(eventList) && (c = XisbRead(pMse->buffer)) >= 0)
-	pBuf[n++] = (unsigned char)c;
+        pBuf[n++] = (unsigned char)c;
 
     if (n == remainder)
-	return;
+        return;
 
     remainder = n % sizeof(kd_event);
     n /= sizeof(kd_event);
     while( n-- ) {
-	int buttons = pMse->lastButtons;
-	int dx = 0, dy = 0;
-	switch (event->type) {
-	case MOUSE_RIGHT:
-	    buttons  = (buttons & 6) |(event->value.up ? 0 : 1);
-	    break;
-	case MOUSE_MIDDLE:
-	    buttons  = (buttons & 5) |(event->value.up ? 0 : 2);
-	    break;
-	case MOUSE_LEFT:
-	    buttons  = (buttons & 3) |(event->value.up ? 0 : 4) ;
-	    break;
-	case MOUSE_MOTION:
-	    dx = event->value.mmotion.mm_deltaX;
-	    dy = - event->value.mmotion.mm_deltaY;
-	    break;
-	default:
-	    ErrorF("Bad mouse event (%d)\n",event->type);
-	    continue;
-	}
-	pMse->PostEvent(pInfo, buttons, dx, dy, 0, 0);
-	++event;
+        int buttons = pMse->lastButtons;
+        int dx = 0, dy = 0;
+        switch (event->type) {
+        case MOUSE_RIGHT:
+            buttons  = (buttons & 6) |(event->value.up ? 0 : 1);
+            break;
+        case MOUSE_MIDDLE:
+            buttons  = (buttons & 5) |(event->value.up ? 0 : 2);
+            break;
+        case MOUSE_LEFT:
+            buttons  = (buttons & 3) |(event->value.up ? 0 : 4) ;
+            break;
+        case MOUSE_MOTION:
+            dx = event->value.mmotion.mm_deltaX;
+            dy = - event->value.mmotion.mm_deltaY;
+            break;
+        default:
+            LogMessageVerbSigSafe(X_ERROR, -1, "Bad mouse event (%d)\n",event->type);
+            continue;
+        }
+        pMse->PostEvent(pInfo, buttons, dx, dy, 0, 0);
+        ++event;
     }
     memcpy(eventList, event, remainder);
     return;
@@ -144,27 +144,19 @@ OsMousePreInit(InputInfoPtr pInfo, const char *protocol, int flags)
     xf86Msg(X_CONFIG, "%s: Protocol: %s\n", pInfo->name, protocol);
 
     /* Collect the options, and process the common options. */
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 12
-    /* need some special handling here. xf86CollectInputOptions will reset
-     * pInfo->options. To re-merge the previously set arguments, pass the
-     * original pInfo->options in.
-     */
-    xf86CollectInputOptions(pInfo, NULL, pInfo->options);
-#else
-    COLLECT_INPUT_OPTIONS(pInfo, NULL);
-#endif
+    xf86CollectInputOptions(pInfo, NULL);
     xf86ProcessCommonOptions(pInfo, pInfo->options);
 
     /* Check if the device can be opened. */
-    pInfo->fd = xf86OpenSerial(pInfo->options); 
+    pInfo->fd = xf86OpenSerial(pInfo->options);
     if (pInfo->fd == -1) {
-	if (xf86GetAllowMouseOpenFail())
-	    xf86Msg(X_WARNING, "%s: cannot open input device\n", pInfo->name);
-	else {
-	    xf86Msg(X_ERROR, "%s: cannot open input device\n", pInfo->name);
-	    free(pMse);
-	    return FALSE;
-	}
+        if (xf86GetAllowMouseOpenFail())
+            xf86Msg(X_WARNING, "%s: cannot open input device\n", pInfo->name);
+        else {
+            xf86Msg(X_ERROR, "%s: cannot open input device\n", pInfo->name);
+            free(pMse);
+            return FALSE;
+        }
     }
     xf86CloseSerial(pInfo->fd);
     pInfo->fd = -1;
@@ -174,10 +166,7 @@ OsMousePreInit(InputInfoPtr pInfo, const char *protocol, int flags)
 
     /* Setup the local procs. */
     pInfo->read_input = OsMouseReadInput;
-    
-#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 12
-    pInfo->flags |= XI86_CONFIGURED;
-#endif
+
     return TRUE;
 }
 
@@ -190,13 +179,13 @@ FindDevice(InputInfoPtr pInfo, const char *protocol, int flags)
     SYSCALL (fd = open(path, O_RDWR | O_NONBLOCK | O_EXCL));
 
     if (fd == -1)
-	return NULL;
+        return NULL;
 
     close(fd);
     pInfo->options =
-	xf86AddNewOption(pInfo->options, "Device", path);
+        xf86AddNewOption(pInfo->options, "Device", path);
     xf86Msg(X_INFO, "%s: Setting Device option to \"%s\"\n", pInfo->name,
-	    path);
+            path);
 
     return path;
 }
@@ -209,8 +198,8 @@ SupportedInterfaces(void)
 }
 
 static const char *internalNames[] = {
-	"OSMouse",
-	NULL
+        "OSMouse",
+        NULL
 };
 
 static const char **
@@ -225,8 +214,8 @@ CheckProtocol(const char *protocol)
     int i;
 
     for (i = 0; internalNames[i]; i++)
-	if (xf86NameCmp(protocol, internalNames[i]) == 0)
-	    return TRUE;
+        if (xf86NameCmp(protocol, internalNames[i]) == 0)
+            return TRUE;
     return FALSE;
 }
 
@@ -241,9 +230,9 @@ OSMouseInit(int flags)
 {
     OSMouseInfoPtr p;
 
-    p = calloc(sizeof(OSMouseInfoRec), 1);
+    p = calloc(1, sizeof(OSMouseInfoRec));
     if (!p)
-	return NULL;
+        return NULL;
     p->SupportedInterfaces = SupportedInterfaces;
     p->BuiltinNames = BuiltinNames;
     p->FindDevice = FindDevice;
